import wx
scene=poser.Scene()

# This function is used for debuging
def showData(data):
    wx.MessageBox(parent=None,message=str(data),style=wx.OK|wx.CENTER)

    return

# This function strips figure names as done by the Fab processes and adds a suffix to return a key to data
def createDataKey(figNm,suffix):
    dpKey=str(figNm).replace(' ','')+suffix

    return dpKey


# This function reads data from a Studio Martillo data prop. It takes a reference to the data prop
# and a keyword and returns the data
def readDataProp(datProp,dpKey):
    data=datProp.CustomData(dpKey)
    
    return data


# This function looks for the HybridFigure factory prop in the scene and returns a reference to it if found
def initBuild():
    factProp=None
    targFigName=None
    gfActors=scene.Actors()
    for gfAct in gfActors:
        if gfAct.IsProp():
            targFigName=gfAct.CustomData('targetfigure')
            if isinstance(targFigName, unicode):
                factProp=gfAct
                break

    return (factProp,targFigName)

# This function iterates the body part actors in the figure, finds the ones that match in the HybridFigure factory prop,
# and makes corresponding lists. One of the internal names, and one of the hidden polygon indices saved in
# the HybridFigure factory prop and returns a list of the lists
def getHiddenMatActors(trgFigure):
    hmActors=[]
    hmGroups=[]
    trgActors=trgFigure.Actors()
    for act in trgActors:
        if act.IsBodyPart():
            actIntName=act.InternalName()
            if ":" in actIntName:
                endex=actIntName.index(":")
                if endex > 0:
                    actIntName=actIntName[0:endex]

            hdKey=createDataKey(actIntName,'hidden')
            fctData=readDataProp(factoryProp,hdKey)
            if fctData:
                hmActors.append(actIntName)
                hmGroups.append(fctData)

    hidMatData=[hmActors,hmGroups]

    return hidMatData

# This function uses the path passed to it to load the HybridFigure parts saved as an assembly and
# returns a tuple containing a reference to the figure and it's name
def loadSubFigure(path):
    subFig=None
    subFigName=None
    scene.LoadLibraryFigure(path)
    subFig=scene.CurrentFigure()
    subFigName=subFig.Name()
    subFigure=(subFig,subFigName)

    return subFigure

# This function sets the actor passed to it and all of the actor's children invisible
def hideActors(parentActor):
    families=[]
    m=0
    n=0
    parentActor.SetVisible(0)
    families.append(parentActor.Children())
    morechildren=True
    while morechildren:
        if n>m:
            morechildren=False
            break
        else:
            children=families[n]
            n=n+1
            for child in children:
                child.SetVisible(0)
                nextgen=child.Children()
                if len(nextgen)>0:
                    families.append(nextgen)
                    m=m+1
    return

# This function takes the lists of actors and hidden polygon indices, and a reference to the figure and
# reassigns the polygons in the lists to the 'Hidden' material
def changeMatGroups(matActorPolys,cmFigure):
    cmActors=matActorPolys[0]
    cmPolyLists=matActorPolys[1]
    i=0
    for actName in cmActors:
        act = cmFigure.ActorByInternalName(actName)
        cmGeom=act.Geometry()
        cmPolys=cmPolyLists[i]
        i=i+1
        if len(cmPolys)>0:
            for plyIndex in cmPolys:
                poly=cmGeom.Polygon(plyIndex)
                poly.SetMaterialName('Hidden')
    return

#This function gets a list of actors to be set invisible from the HybridFigure factory prop and calls the hideActors function
# for each one with a scene reference argument
def setInvisibleActors(siFigure):
    siTopNames=factoryProp.CustomData('invisible')
    for topname in siTopNames:
        topact=siFigure.ActorByInternalName(topname)
        hideActors(topact)

    return

# This function takes the target figure name and gets the path from the HybridFigure factory prop and
# gets the list of conforming figure names from the HybridFigure factory prop and their paths
# and loads them from the poser library.
# It then sets each one to conform to the target figure and returns a list of conforming figure names
def buildAssembly(trgFgName):
    cfNameList=[]
    tfFileKey=createDataKey(trgFgName,'file')
    targFigFile=readDataProp(factoryProp,tfFileKey)
    scene.LoadLibraryFigure(targFigFile)
    targetFigure=scene.Figure(trgFgName)
    conFigures=readDataProp(factoryProp,'conformfigures')
    if conFigures:
        for cfName in conFigures:
            cfKey=createDataKey(cfName,'file')
            cfPath=readDataProp(factoryProp,cfKey)
            if cfPath:
                scene.LoadLibraryFigure(cfPath)
                conFig=scene.CurrentFigure()
                conFig.SetConformTarget(targetFigure)
                cfNameList.append(cfName)
                    
    return (targetFigure,cfNameList)


# This function takes a scene figure name and gets the material collection filename
# string from the HybridFigureFactory prop. It then loads the material collection to the 'Body' actor
# of the named figure
def loadFigureMaterials(lmFigName):
    matKey=createDataKey(lmFigName,'mats')
    matFile=readDataProp(factoryProp,matKey)
    lmFigure=scene.Figure(lmFigName)
    lmBody=lmFigure.Actor('Body')
    lmBody.LoadMaterialCollection(matFile)
    
    return


# START OF MAIN SCRIPT _____________________________________________>>
#_______________________________________________________________________>>
principles=initBuild()
if principles:
    factoryProp=principles[0]
    targFigName=principles[1]
    targConfNames=buildAssembly(targFigName)
    targFig=targConfNames[0]
    confFigures=targConfNames[1]
    loadFigureMaterials(targFigName)
    for cnFg in confFigures:
        loadFigureMaterials(cnFg)

    matDataLists=getHiddenMatActors(targFig)
    changeMatGroups(matDataLists,targFig)
    setInvisibleActors(targFig)
    scene.SelectActor(factoryProp)
    scene.DeleteCurrentProp()
else:
    wxMessageBox(parent=None,message="Principle objects not found in factory prop.",style=wx.OK|wx.CENTER)

