import wx

scene=poser.Scene()

# This function is used for debuging
def showData(data):
    wx.MessageBox(parent=None,message=str(data),style=wx.OK|wx.CENTER)

    return


# This function loads the factory prop from poser runtime props library
def loadFactoryProp():
    scene.LoadLibraryProp(":Runtime:Libraries:Props:Studio Martillo:HybridFigure:FigureFactory.pp2")
    fctProp=scene.CurrentActor()
    return fctProp


# This function prompts for a product name for the new Hybrid Figure
def askProductName():
    prdNameDlg = wx.TextEntryDialog(parent=None,message='Please Enter the Product Name')
    prdNameDlg.ShowModal()
    productName = prdNameDlg.GetValue()
    prdNameDlg=None

    return productName


# This function writes data to a Studio Martillo data prop. It takes a reference to the data prop
# and a keyword and data object

def writeDataProp(datProp,key,data):
    dpKey=str(key).replace(' ','')
    datProp.SetCustomData(dpKey,data,0,0)

    return

# This function takes a reference to a figure in the scene and presents a file selector to get
# the path to the figure's file. 
def selectFigureFile(sfFigure):
    figName=sfFigure.Name()
    figFile=wx.FileSelector("Select the Poser Library file for "+figName,wildcard="Character files | *.cr2")
    if len(figFile)>7:
        figFile=figFile.replace('\\',':')
        stSplt=figFile.find(':Runtime')
        stEnd=len(figFile)
        figFile=figFile[stSplt:stEnd]
    else:
        figFile=None

    return figFile


# This function takes a reference to the Hybrid Figure target figure and saves the name
# in the data prop with the same formatting as the conforming figures and keyword 'targetfigure'
# and also prompts for a file name and saves the filename in the data prop with keyword as above
def saveTargetFigure(targFig):
    targFile=selectFigureFile(targetFigure)
    if targFile:
        writeDataProp(hybridFigureFactory,targetFigure.Name()+"file",targFile)
        targName=targFig.Name()
        n=len(targName)
        endChar=targName[n-1]
        if endChar.isnumeric():
            targName=targName[0:n-2]

        writeDataProp(hybridFigureFactory,'targetfigure',targName)
        stat=True

    else:
        stat=None

    return stat


# This function iterates the list of figures in the scene and makes a list of the ones that are conforming
# and a list of the conforming target and returns a tuple of the lists
def getConformingFigures():
    conformFigures=[]
    targFigures=[]
    figures=scene.Figures()
    for fig in figures:
        targ=fig.ConformTarget()
        if targ:
            targFigures.append(targ)
            conformFigures.append(fig)

    figureLists=(targFigures,conformFigures)

    return figureLists

# This function takes the list of conforming figures and makes a list of their names for data keys and prompts for
# the libary file which it adds as data and saves as custom data in the hybridFigureFactory prop
def saveConformingFigureList(conFigs):
    cfNames=[]
    stat=True
    for fig in conFigs:
        figName=fig.Name()
        n=len(figName)
        endChar=figName[n-1]
        if endChar.isnumeric():
            figName=figName[0:n-2]

        cfNames.append(figName)
        figFile=selectFigureFile(fig)
        if not figFile:
            stat=None
            break

        else:
            writeDataProp(hybridFigureFactory,figName+"file",figFile)

    if stat:
        writeDataProp(hybridFigureFactory,'conformfigures',cfNames)

    return stat

# This function gets the geometry object from the given actor and makes a list of the polygons
# assigned to the "Hidden" material    
def getHiddenPolys(actor):
    hiddenPolys=[]
    geom=actor.Geometry()
    if geom:
        mats=geom.Materials()
        numPolys=geom.NumPolygons()
        hasHidden=False
        for mat in mats:
            name=mat.ExtName()
            if 'Hidden' in name:
                hasHidden=True

        if hasHidden:
            index=0
            while index < numPolys:
                poly=geom.Polygon(index)
                if 'Hidden' in poly.MaterialName():
                    hiddenPolys.append(index)
                index=index+1

    return hiddenPolys

# This function takes a figure and gets the list of actors. For each body part actor it calls getHiddenPolys()
# to get the list of 'hidden' polygons and gets the internal name of the body part
# they belong to and saves it to custom data in the hybridFigure factory prop
def saveHiddenPolyData(hpFigure):
    hpActorNames=[]
    figActors=hpFigure.Actors()
    for figActor in figActors:
        if figActor.IsBodyPart():
            faIntName=figActor.InternalName()
            if ":" in faIntName:
                endex=faIntName.index(":")
                if endex > 0:
                    faIntName=faIntName[0:endex]
            hidPolygons=getHiddenPolys(figActor)
            if len(hidPolygons)>0:
                writeDataProp(hybridFigureFactory,faIntName+"hidden",hidPolygons)
                hpActorNames.append(faIntName)

    writeDataProp(hybridFigureFactory,'hiddenmatactors',hpActorNames)

    return

# This function iterates the body parts of the figure and makes a list of the ones that are set invisible
def getInvisibleActors(parFigure):
    invActs=[]
    pfActors=parFigure.Actors()
    for actr in pfActors:
        if actr.IsBodyPart():
            if not actr.Visible():
                invActs.append(actr)

    return invActs

# This function calls getInvisibleActors() to get the list of invisible body parts
# and saves it to custom data in the hybridFigure factory prop
def saveInvisibleActors(iaFigure):
    actIntNames=[]
    actLst=getInvisibleActors(iaFigure)
    for act in actLst:
        actName=act.InternalName()
        if not "BODY" in actName:
            if ":" in actName:
                endex=actName.index(":")
                if endex > 0:
                    actName=actName[0:endex]
            actIntNames.append(actName)

    writeDataProp(hybridFigureFactory,"invisible",actIntNames)

    return


# This function takes a reference to a figure in the scene and presents a file selector to get
# the path to a Material Collection. 
def selectMaterialCollection(matFigure):
    figName=matFigure.Name()
    mtrlCollFile=wx.FileSelector(message="Select a Material Collection for "+figName,wildcard="Material Collection | *.mc6")
    if len(mtrlCollFile)>7:
        mtrlCollFile=mtrlCollFile.replace('\\',':')
        stSplt=mtrlCollFile.find(':Runtime')
        stEnd=len(mtrlCollFile)
        mtrlCollFile=mtrlCollFile[stSplt:stEnd]
    else:
        mtrlCollFile=None

    return mtrlCollFile


# This function takes a Material Collection path&Name and a reference to a figure in the scene,
# saves the collection info to the hybridFigureFactory prop and applies the collection to the 'Body'
# actor of the figure
def saveMaterialCollection(matFile,matFigure):
    strpName=str(matFigure.Name()).replace(' ','')
    n=len(strpName)
    endChar=strpName[n-1]
    if endChar.isdigit():
        strpName=strpName[0:n-2]
    mfKey=strpName+'mats'
    writeDataProp(hybridFigureFactory,mfKey,matFile)
    figBdy=matFigure.Actor('Body')
    figBdy.LoadMaterialCollection(matFile)

    return


# START OF MAIN SCRIPT _____________________________________________>>
#_______________________________________________________________________>>

hybridFigureFactory=loadFactoryProp()
productName=askProductName()
hybridFigureFactory.SetName(productName)
factdone=True
confFigList=getConformingFigures()
targFigSet=set(confFigList[0])
numTargs=len(targFigSet)

if numTargs != 1:
    wx.MessageBox(parent=None,message="There must be one target figure for all conforming figures in the scene.",style=wx.OK|wx.CENTER)
else:
    conformingFigures=confFigList[1]
    if saveConformingFigureList(conformingFigures):
        targetFigure=targFigSet.pop()
        if saveTargetFigure(targetFigure):
            saveInvisibleActors(targetFigure)
            saveHiddenPolyData(targetFigure)
            pfMats=selectMaterialCollection(targetFigure)
            if pfMats:
                saveMaterialCollection(pfMats,targetFigure)
                for mtlFgr in conformingFigures:
                    matCollFile=selectMaterialCollection(mtlFgr)
                    if matCollFile:
                        saveMaterialCollection(matCollFile,mtlFgr)
                    else:
                        factdone=False
                        break
            else:
                factdone=False
        else:
            factdone=False
    else:
        factdone=False
    
    if factdone:        
        wx.MessageBox(parent=None,message='HybridFigure Configuration Written to HybridFigure Factory Prop',style=wx.OK|wx.CENTER)
    else:
        wx.MessageBox(parent=None,message='HybridFigure Configuration Canceled',style=wx.OK|wx.CENTER)
